#!/usr/bin/python

import sys
import os
import re
import random
import fileinput

die_exp = re.compile('d([0-9]+)')
assert die_exp.match('d8')
assert die_exp.match('d20')
assert not die_exp.match('1d8')
assert not die_exp.match('1d8x2')
assert not die_exp.match('1d8+3')
assert not die_exp.match('1d8x2+3')
roll_exp = re.compile('([0-9]+)?d([0-9]+)(?:x([0-9]+))?(?:\\+([0-9]+))?')
assert roll_exp.match('d8')
assert roll_exp.match('d20')
assert roll_exp.match('1d8')
assert roll_exp.match('1d8x2')
assert roll_exp.match('1d8+3')
assert roll_exp.match('1d8x2+3')
_test_match = roll_exp.match('1d8x2+3')
assert _test_match.group(1) == '1'
assert _test_match.group(2) == '8'
assert _test_match.group(3) == '2'
assert _test_match.group(4) == '3'

class BarCharacter(object):
	BLOCK_FULL = u'\u2588'
	BLOCK_7_8 = u'\u2589'
	BLOCK_6_8 = u'\u258A'
	BLOCK_5_8 = u'\u258B'
	BLOCK_4_8 = u'\u258C'
	BLOCK_3_8 = u'\u258D'
	BLOCK_2_8 = u'\u258E'
	BLOCK_1_8 = u'\u258F'
	BLOCK_0_8 = u' '
def print_row(num, count, max_count, BarCharacter=BarCharacter):
	def depict_bar(count, max_count, _graph_length=50, _blocks = [ BarCharacter.BLOCK_0_8, BarCharacter.BLOCK_1_8, BarCharacter.BLOCK_2_8, BarCharacter.BLOCK_3_8, BarCharacter.BLOCK_4_8, BarCharacter.BLOCK_5_8, BarCharacter.BLOCK_6_8, BarCharacter.BLOCK_7_8, BarCharacter.BLOCK_FULL ]):
		fraction = float(count) / max_count
		num_full = _graph_length * fraction
		remainder = (_graph_length * fraction - int(num_full)) * 8
		return int(num_full) * BarCharacter.BLOCK_FULL + _blocks[int(remainder)]
	print '%3u\t%s %u' % (num, depict_bar(count, max_count), count)
del BarCharacter
if False:
	#Testing
	print_row(0, 0, 500)
	print_row(1, 50, 500)
	print_row(2, 490, 500)
	print_row(2, 492, 500)
	print_row(3, 499, 500)
	print_row(4, 500, 500)

num_rolls = 10000

def gen_die_rolls(die):
	match = die_exp.match(die)
	assert match, 'This is not a valid die specification: ' + repr(die)

	filename = die + '.rolls'
	with open(filename, 'w') as f:
		num_faces = int(match.group(1))
		faces = range(1, num_faces + 1)

		for x in xrange(num_rolls):
			print >>f, random.choice(faces)

def load_die_rolls(die):
	filename = die + '.rolls'
	if not os.path.exists(filename):
		gen_die_rolls(die)
	with open(filename, 'r') as f:
		return [ int(x) for x in f ]

def roll(num_dice, die_rolls):
	die_rolls = list(die_rolls)
	while die_rolls:
		rolled = die_rolls[:num_dice]
		del die_rolls[:num_dice]
		yield rolled

for roll_spec in sys.argv[1:]:
	match = roll_exp.match(roll_spec)
	num_dice, num_faces, roll_multi, addend = match.groups()
	if num_dice is None: num_dice = 1
	if roll_multi is None: roll_multi = 1
	if addend is None: addend = 0

	num_dice = int(num_dice)
	num_faces = int(num_faces)
	roll_multi = int(roll_multi)
	addend = int(addend)

	die = 'd' + str(num_faces)
	die_rolls = load_die_rolls(die)

	resultant_totals = []
	for results in roll(num_dice, die_rolls):
		resultant_totals.append(sum(results) * roll_multi + addend)

	counts = {}
	max_count = 0
	for x in resultant_totals:
		counts.setdefault(x, 0)
		count = counts[x] + 1
		counts[x] = count
		if count > max_count: max_count = count
	buckets = counts.keys()
	buckets.sort()

	for face in buckets:
		try:
			count = counts[face]
		except IndexError:
			print >>sys.stderr, face
			raise
		print_row(face, count, max_count)
